/*
 * Copyright (c) 2011-2019, Peter Abeles. All Rights Reserved.
 *
 * This file is part of BoofCV (http://boofcv.org).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package boofcv.alg.feature.detect.intensity.impl;

import boofcv.alg.feature.detect.intensity.HarrisCornerIntensity;

/**
 * <p>
 * Implementation of {@link boofcv.alg.feature.detect.intensity.HarrisCornerIntensity}.
 * </p>
 *
 * @author Peter Abeles
 */
public class HarrisCorner_S32 implements ImplSsdCornerBase.CornerIntensity_S32 , HarrisCornerIntensity {

	// tuning parameter
	float kappa;

	public HarrisCorner_S32(float kappa) {
		this.kappa = kappa;
	}

	@Override
	public void setKappa(float kappa) {
		this.kappa = kappa;
	}

	@Override
	public float compute( int _totalXX, int _totalXY, int _totalYY ) {
		// det(A) -+ kappa*trace(A)^2
		float totalXX = _totalXX, totalYY = _totalYY, totalXY = _totalXY;
		float trace = totalXX + totalYY;

		return (totalXX * totalYY - totalXY * totalXY) - kappa * trace*trace;
	}

	@Override
	public float getKappa() {
		return kappa;
	}
}
